/*
 * Get canonical name of the host
 */

#ifndef lint
char rcsid[] = "@(#)netname.c,v 1.14 2011/08/25 18:00:21 kim Exp";
#endif

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/utsname.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#ifdef ultrix
#include <sys/svcinfo.h>
#endif

void usage(char *me)
{
  fprintf(stderr, "Usage: %s [-a] [-d] [-v] [host]\n", me);
  exit(1);
}

int main(int argc, char **argv)
{
  char *host = NULL;
  struct utsname name;
  struct addrinfo hints, *res = NULL, *rp;
  int a = 0, c, d = 0, e;
#ifdef ultrix
  struct svcinfo *svcinfo;
#endif
  extern int optind;

  while ((c = getopt(argc, argv, "adv")) != EOF)
    switch (c) {
    case 'a':
      a = 1;
      break;
    case 'd':
      d = 1;
      break;
    case 'v':
      fprintf(stderr,
	      "%s version 1.9 (Copyright (c) 1995-2011 Kimmo Suominen)\n%s\n",
	      argv[0], rcsid);
      /* fall through */
    default:
      usage(argv[0]);
    }

  switch (argc - optind) {
  case 0:
    if (uname(&name) == -1) {
      fprintf(stderr, "%s: cannot get uname information\n", argv[0]);
      exit(2);
    }
    host = name.nodename;
    break;
  case 1:
    host = argv[optind];
    break;
  default:
    usage(argv[0]);
  }

#ifdef ultrix
  /* On ultrix, force use of the DNS resolver instead of the hosts file */
  if ((svcinfo = getsvc()) != NULL) {
    if (svcinfo->svcpath[SVC_HOSTS][0] == SVC_LOCAL) {
      int i;

      for (i = 0; i < SVC_PATHSIZE - 1; i++)
        svcinfo->svcpath[SVC_HOSTS][i] = svcinfo->svcpath[SVC_HOSTS][i + 1];
    }
  }
#endif

  memset(&hints, 0, sizeof(hints));
  hints.ai_family = AF_UNSPEC;
  hints.ai_flags = AI_CANONNAME;

  /*
   * Without the socktype hint, the returned list will have
   * an entry for each possible socktype for each address.
   */

  hints.ai_socktype = SOCK_STREAM;

  e = getaddrinfo(host, NULL, &hints, &res);
  if (e) {
    fprintf(stderr, "%s: %s: %s\n", argv[0], host, gai_strerror(e));
    exit(1);
  }

  if (d && (host = strchr(res->ai_canonname, '.')) != NULL)
    host++;
  else
    host = res->ai_canonname;

  if (a) {
    printf("Canonical hostname is %s\n", host);
    for (rp = res; rp != NULL; rp = rp->ai_next) {
      char h[NI_MAXHOST] = "";
      char p[NI_MAXHOST] = "";

      switch (rp->ai_addr->sa_family) {
	case AF_INET:
	  inet_ntop(AF_INET,
		    &(((struct sockaddr_in *)rp->ai_addr)->sin_addr),
		    p, NI_MAXHOST);
	  break;
	case AF_INET6:
	  inet_ntop(AF_INET6,
		    &(((struct sockaddr_in6 *)rp->ai_addr)->sin6_addr),
		    p, NI_MAXHOST);
	  break;
	default:
	  strncpy(p, "<unprintable>", NI_MAXHOST);
	  break;
      }

      e = getnameinfo(rp->ai_addr, rp->ai_addrlen, h, NI_MAXHOST, NULL, 0, 0);
      if (e) {
	fprintf(stderr, "%s: %s: address %s: %s\n",
		argv[0], host, p, gai_strerror(e));
      }
      if (*h != '\0')
	printf("Address %s points to %s\n", p, h);
    }
  } else {
    printf("%s\n", host);
  }

  freeaddrinfo(res);
  return(0);
}
